@@ -1,3 +1,6 @@
+// compile with make LOAD_FROM_FILES=1 DUMP=1
+// to dump Wavs
+
 #ifdef _WIN32
 #include <direct.h>
 #else
@@ -15,21 +18,19 @@
 #endif
 
 #ifdef LOAD_FROM_FILES
-#include "streams/file_stream.h"
+#include <streams/file_stream.h>
 #include <minizip/unzip.h>
 #endif
 
+#ifndef NO_NETWORK
 extern "C" {
 #include <net/net_http.h>
 #include <net/net_compat.h>
 }
-
-#define SOUND_VOLUME                        2
-#ifdef __LIBSDL2__
-#define NB_WAV                              21
-#else
-#define NB_WAV                              16
 #endif
+
+#define SOUND_VOLUME                        1
+#define NB_WAV                              21
 #define NB_VOICES                           28
 #define keyboardCodeOffset                  32
 #define keyboardReturnKey                   28
@@ -43,8 +44,19 @@ extern "C" {
 #pragma GCC diagnostic ignored "-Woverlength-strings"
 #pragma GCC diagnostic ignored "-Warray-bounds"
 
+#define NB_CHIPTUNES    8
+
 #ifdef __LIBRETRO__
+#include <audio/audio_mixer.h>
+#include <audio/conversion/float_to_s16.h>
+static float *              fbuf = NULL;
+static int16_t *            ibuf = NULL;
+static audio_mixer_sound_t *musics[NB_CHIPTUNES];
+#ifndef LOAD_FROM_FILES
 #include "retro_data.h"
+#include "retro_music_data.h"
+#endif
+
 #include "retro.hpp"
 
 #ifdef LOAD_FROM_FILES
@@ -65,16 +77,8 @@ static size_t frames_left[NB_WAV];
 #define CLAMP_I16(x)    (x > INT16_MAX ? INT16_MAX : x < INT16_MIN ? INT16_MIN : x)
 #endif
 
-#ifdef __LIBSDL2__
-#include "sdl2_data.h"
-#define LOAD_FROM_FILES
-#include <SDL2/SDL.h>
-#include <SDL2/SDL_mixer.h>
-static Mix_Chunk * wave[NB_WAV];
-#define NB_CHIPTUNES    8
-static Mix_Music *musics[NB_CHIPTUNES];
-static int        musics_index = 0;
-bool        music = true;
+bool music = true;
+static int  musics_index = 0;
 const char *musics_filenames[NB_CHIPTUNES] = {
    "deadfeelings.XM",                         // Carter (for menu + replay)
    "chiptune.MOD",                            // 4-mat
@@ -85,6 +89,13 @@ const char *musics_filenames[NB_CHIPTUNES] = {
    "external.XM",                             // Quazar
    "ESTRAYK-Drop.MOD"                         // Estrayk
 };
+#ifdef __LIBSDL2__
+#define LOAD_FROM_FILES
+#include <SDL2/SDL.h>
+#include <SDL2/SDL_mixer.h>
+static Mix_Chunk *wave[NB_WAV];
+static Mix_Music *musics[NB_CHIPTUNES];
+
 #define DEFAULT_VOLUME     MIX_MAX_VOLUME / 2
 #define MATKAMIE_VOLUME    MIX_MAX_VOLUME
 #define LOWER_VOLUME       MIX_MAX_VOLUME / 3
@@ -100,7 +111,16 @@ const int musics_volume[NB_CHIPTUNES] = {
    LOWER_VOLUME
 };
 #endif
+
+
+#ifdef LOAD_FROM_FILES
+#include "sdl2_data.h"
+#endif
+
+#ifndef NO_NETWORK
 static bool network_init_done = false;
+#endif
+
 static int  ignoreForAbit[NB_WAV];
 static int  ignoreForAbitFlag[NB_WAV];
 int         traceMask    = DEFAULT_TRACE_MAX;
@@ -126,7 +146,6 @@ int walkingToCell[nb_dyna];
 int rom_unzip(const char *path, const char *extraction_directory)
 {
    path_mkdir(extraction_directory);
-
    unzFile *zipfile = (unzFile *)unzOpen(path);
    if (zipfile == NULL)
    {
@@ -290,6 +309,11 @@ bool mrboom_init()
    }
    strcpy((char *)&m.iff_file_name, "mrboom.dat");
    m.taille_exe_gonfle = 0;
+#ifdef __LIBRETRO__
+   fbuf = (float *)malloc(num_samples_per_frame * 2 * sizeof(float));
+   ibuf = (int16_t *)malloc(num_samples_per_frame * 2 * sizeof(int16_t));
+   audio_mixer_init(SAMPLE_RATE);
+#endif
 #ifdef __LIBSDL2__
    /* Initialize SDL. */
    if (SDL_Init(SDL_INIT_AUDIO) < 0)
@@ -303,15 +327,6 @@ bool mrboom_init()
       log_error("Error Mix_OpenAudio\n");
       audio = false;
    }
-   const char *scrolltext = "  players can join the game using their action keys...   use the b button (pad) or ctrl to drop a bomb   a (pad) or alt to trigger the bomb remote control   x (pad) or shift to jump with a kangaroo   select (pad) or space to add a bomber-bot   start (pad) or return to start!   check the command lines options to enable team modes   graphics by zaac exocet easy and marblemad   musics by 4-mat carter heatbeat quazar jester estrayk rez and kenet  (c) 1997-2018 remdy software.     (wrap time)  "
-   ;
-#ifdef DEBUG
-   int sizeBuffer = offsetof(struct Mem, message1) - offsetof(struct Mem, tecte);
-   log_info("%d %d\n", strlen(scrolltext), sizeBuffer);
-   assert((strlen(scrolltext) + 1) < sizeBuffer);
-#endif
-   strcpy((char *)m.tecte, scrolltext);
-   m.tecte[strlen((char *)m.tecte)] = 219;
 #endif
    m.tected[20] = GAME_VERSION[0];
    m.tected[21] = GAME_VERSION[1];
@@ -348,29 +363,36 @@ bool mrboom_init()
       log_error("Error writing %s\n", romPath);
       return(false);
    }
+
    rom_unzip(romPath, extractPath);
+
    unlink(romPath);
    m.path = strdup(extractPath);
-   char tmp[PATH_MAX_LENGTH];
+   char filePath[PATH_MAX_LENGTH];
 #endif
    for (int i = 0; i < NB_WAV; i++)
    {
 #ifdef LOAD_FROM_FILES
-      sprintf(tmp, "%s/%d.WAV", extractPath, i);
+      sprintf(filePath, "%s/%d.WAV", extractPath, i);
 #ifdef __LIBRETRO__
-      wave[i] = audio_mix_load_wav_file(&tmp[0], SAMPLE_RATE);
+      wave[i] = audio_mix_load_wav_file(&filePath[0], SAMPLE_RATE);
 #endif
 #ifdef __LIBSDL2__
       if (audio)
       {
-         wave[i] = Mix_LoadWAV(tmp);
+         wave[i] = Mix_LoadWAV(filePath);
+         if (wave[i] == NULL)
+         {
+            log_error("Couldn't load %s\n", filePath);
+            exit(1);
+         }
          Mix_VolumeChunk(wave[i], MIX_MAX_VOLUME * sdl2_fx_volume / 10);
       }
+      unlink(filePath);
 #endif
-      unlink(tmp);
       if (wave[i] == NULL)
       {
-         log_error("cant load %s\n", tmp);
+         log_error("cant load %s\n", filePath);
       }
 #endif
       ignoreForAbit[i]     = 0;
@@ -379,18 +401,46 @@ bool mrboom_init()
 #ifdef __LIBSDL2__
    for (int i = 0; i < NB_CHIPTUNES; i++)
    {
-      sprintf(tmp, "%s/%s", extractPath, musics_filenames[i]);
+      sprintf(filePath, "%s/%s", extractPath, musics_filenames[i]);
       if (audio)
       {
-         musics[i] = Mix_LoadMUS(tmp);
+         musics[i] = Mix_LoadMUS(filePath);
          if (!musics[i])
          {
             log_error("Mix_LoadMUS(\"%s\"): %s: please check SDL2_mixer is compiled --with-libmodplug\n", musics_filenames[i], Mix_GetError());
             return(false);
          }
       }
-      unlink(tmp);
+      unlink(filePath);
    }
+#endif
+#ifdef __LIBRETRO__
+#ifdef LOAD_FROM_FILES
+   for (int i = 0; i < NB_CHIPTUNES; i++)
+   {
+      sprintf(filePath, "%s/%s", extractPath, musics_filenames[i]);
+      int64_t len = 0;
+      void *  buf = NULL;
+      if (!filestream_read_file(filePath, &buf, &len))
+      {
+         log_error("Could not load %s\n", filePath);
+         musics[i] = NULL;
+      }
+      else
+      {
+         musics[i] = audio_mixer_load_mod(buf, len);
+      }
+   }
+#else
+   musics[0] = audio_mixer_load_mod(rom_deadfeelings_XM, rom_deadfeelings_XM_len);
+   musics[1] = audio_mixer_load_mod(rom_chiptune_MOD, rom_chiptune_MOD_len);
+   musics[2] = audio_mixer_load_mod(rom_matkamie_MOD, rom_matkamie_MOD_len);
+   musics[3] = audio_mixer_load_mod(rom_jester_chipmunks_MOD, rom_jester_chipmunks_MOD_len);
+   musics[4] = audio_mixer_load_mod(rom_unreeeal_superhero_3_looping_version_XM, rom_unreeeal_superhero_3_looping_version_XM_len);
+   musics[5] = audio_mixer_load_mod(rom_anar11_MOD, rom_anar11_MOD_len);
+   musics[6] = audio_mixer_load_mod(rom_external_XM, rom_external_XM_len);
+   musics[7] = audio_mixer_load_mod(rom_ESTRAYK_Drop_MOD, rom_ESTRAYK_Drop_MOD_len);
+#endif
 #endif
 
    ignoreForAbitFlag[0]  = 30;
@@ -432,15 +482,28 @@ bool mrboom_init()
    {
       tree[i] = new BotTree(i);
    }
+
+#ifndef NO_NETWORK
    if (network_init())
    {
       network_init_done = true;
    }
+#endif
    return(true);
 }
 
 void mrboom_deinit()
 {
+#ifdef __LIBRETRO__
+   for (int i = 0; i < NB_CHIPTUNES; i++)
+   {
+#ifdef LOAD_FROM_FILES
+      audio_mixer_destroy(musics[i]);
+#else
+      free(musics[i]);
+#endif
+   }
+#endif
 #ifdef LOAD_FROM_FILES
    /* free WAV */
    for (int i = 0; i < NB_WAV; i++)
@@ -453,14 +516,22 @@ void mrboom_deinit()
 #endif
    }
 #endif
+#ifdef __LIBRETRO__
+   free(fbuf);
+   free(ibuf);
+   audio_mixer_done();
+#endif
+#ifndef NO_NETWORK
    if (network_init_done)
    {
       network_deinit();
    }
+#endif
 }
 
 static void mrboom_api()
 {
+#ifndef NO_NETWORK
    static struct http_connection_t *conn = NULL;
    static struct http_t *           http = NULL;
    static int api_state    = 0;
@@ -476,7 +547,7 @@ static void mrboom_api()
    {
    case 0:
       char body[1024];
-      sprintf(body, "state=%d&platform=%s&team=%d&version=", currentState, GAME_PLATFORM, teamMode());
+      sprintf(body, "state=%d&platform=%s&team=%d&version=", currentState, PLATFORM, teamMode());
       #ifdef __LIBSDL2__
       strcat(body, "SDL2-");
       #else
@@ -527,14 +598,25 @@ static void mrboom_api()
       }
       break;
    }
+#endif
 }
 
+#ifdef __LIBSDL2__
+#define play(b)    Mix_PlayChannel(-1, wave[b], 0)
+#else
+#ifdef LOAD_FROM_FILES
+#define play(b)    if (wave[b] != NULL) { frames_left[b] = audio_mix_get_chunk_num_samples(wave[b]); }
+#else
+#define play(b)    if (wave[b].samples != NULL) { frames_left[b] = wave[b].num_samples; }
+#endif
+#endif
+
 #define fxSound(a, b)                                 \
    static bool a ## b = false;                        \
    if (a() && !a ## b)                                \
    {                                                  \
       a ## b = true;                                  \
-      Mix_PlayChannel(-1, wave[b], 0);                \
+      play(b);                                        \
       if (fxTraces) { log_debug("fxSound "#a "\n"); } \
    }                                                  \
    a ## b = a();
@@ -545,14 +627,17 @@ void mrboom_sound(void)
    {
       return;
    }
-#ifdef __LIBSDL2__
    fxSound(isDrawGame, 16)
    fxSound(won, 17)
    fxSound(isApocalypseSoon, 18)
    fxSound(isGamePaused, 19)
    fxSound(isGameUnPaused, 5)
    fxSound(playerGotDisease, 20)
+
+#ifdef __LIBRETRO__
+   static audio_mixer_voice_t * voice = NULL;
 #endif
+
    static int last_voice = 0;
    for (int i = 0; i < NB_WAV; i++)
    {
@@ -562,6 +647,19 @@ void mrboom_sound(void)
       }
    }
 
+#ifdef DUMP
+   static bool play_once = true;
+   if (play_once)
+   {
+      play(16);
+      play(17);
+      play(18);
+      play(20);
+      play_once = false;
+   }
+#endif
+
+
    while (m.last_voice != (unsigned)last_voice)
    {
       db a  = *(((db *)&m.blow_what2[last_voice / 2]));
@@ -574,7 +672,7 @@ void mrboom_sound(void)
 #ifdef LOAD_FROM_FILES
       if ((a1 >= 0) && (a1 < NB_WAV) && (wave[a1] != NULL))
 #else
-      if ((a1 >= 0) && (a1 < NB_WAV))
+      if ((a1 >= 0) && (a1 < NB_WAV) && (wave[a1].samples != NULL))
 #endif
       {
          bool dontPlay = 0;
@@ -587,7 +685,6 @@ void mrboom_sound(void)
             }
             dontPlay = 1;
          }
-
          if (dontPlay == 0)
          {
 #ifdef __LIBRETRO__
@@ -623,7 +720,6 @@ void mrboom_sound(void)
          log_error("Wrong sample id %d or NULL.\n", a1);
       }
    }
-#ifdef __LIBSDL2__
    if (music)
    {
       static int currentLevel = -2;
@@ -635,12 +731,23 @@ void mrboom_sound(void)
          {
             index = 0;
          }
+#ifdef __LIBSDL2__
          Mix_VolumeMusic(musics_volume[index]);
          log_debug("Playing %s volume:%d\n", musics_filenames[index], Mix_VolumeMusic(-1));
          if (Mix_PlayMusic(musics[index], -1) == -1)
          {
             log_error("error playing music %d\n", musics[0]);
          }
+#else
+//audio_mixer_voice_t* audio_mixer_play(audio_mixer_sound_t* sound,
+//     bool repeat, float volume, audio_mixer_stop_cb_t stop_cb);
+         if (voice)
+         {
+            audio_mixer_stop(voice);
+         }
+         voice = audio_mixer_play(musics[index], true, 1, NULL);  //stop_cb);
+#endif
+
          if (index)
          {
             musics_index = (musics_index + 1) % (NB_CHIPTUNES);
@@ -651,9 +758,15 @@ void mrboom_sound(void)
          }
       }
    }
-#endif
 }
 
+#ifdef __LIBRETRO__
+void stop_cb(audio_mixer_sound_t *sound, unsigned reason)
+{
+}
+
+#endif
+
 static void mrboom_reset_special_keys()
 {
    db *keys = m.total_t;
@@ -781,10 +894,25 @@ void mrboom_update_input(int keyid, int playerNumber, int state, bool isIA)
 }
 
 #ifdef __LIBRETRO__
+
 void audio_callback(void)
 {
    unsigned i;
 
+#ifdef DUMP
+   static bool dump = true;
+   static bool dumped[NB_WAV];
+
+   if (dump)
+   {
+      for (i = 0; i < NB_WAV; i++)
+      {
+         dumped[i] = true;
+      }
+      dump = false;
+   }
+#endif
+
    if (!audio_batch_cb)
    {
       return;
@@ -801,6 +929,31 @@ void audio_callback(void)
 #ifdef LOAD_FROM_FILES
          int16_t *samples    = audio_mix_get_chunk_samples(wave[i]);
          unsigned num_frames = audio_mix_get_chunk_num_samples(wave[i]);
+#ifdef DUMP
+         FILE *file;
+         if (dumped[i])
+         {
+            char path[PATH_MAX_LENGTH];
+            sprintf(path, "/tmp/audio-%d.c", i);
+            printf("fopen %s\n", path);
+            file = fopen(path, "w");
+            fprintf(file, "static const uint32_t wav%d_data [%u] = {\n", i, num_frames * 2);
+            int nbIntsDumped = 0;
+            while (nbIntsDumped < num_frames * 2)
+            {
+               fprintf(file, "0x%04x, ", samples[nbIntsDumped] | (uint32_t)samples[nbIntsDumped + 1] << 16);
+               nbIntsDumped++;
+               nbIntsDumped++;
+               if (nbIntsDumped % 8 == 0)
+               {
+                  fprintf(file, "\n");
+               }
+            }
+            fprintf(file, "\n};\n");
+            fclose(file);
+            dumped[i] = false;
+         }
+#endif
 #else
          const int16_t *samples    = wave[i].samples;
          unsigned       num_frames = wave[i].num_samples;
@@ -818,6 +971,17 @@ void audio_callback(void)
       }
    }
 
+   memset(fbuf, 0, num_samples_per_frame * 2 * sizeof(float));
+   audio_mixer_mix(fbuf, num_samples_per_frame, 1, false);
+   convert_float_to_s16(ibuf, fbuf, num_samples_per_frame * 2);
+
+   for (i = 0; i < num_samples_per_frame; i++)
+   {
+      frame_sample_buf[i * 2]       = CLAMP_I16(frame_sample_buf[i * 2] + ibuf[i * 2]);
+      frame_sample_buf[(i * 2) + 1] = CLAMP_I16(frame_sample_buf[(i * 2) + 1] + ibuf[(i * 2) + 1]);
+   }
+
+
    audio_batch_cb(frame_sample_buf, num_samples_per_frame);
 }
 
@@ -888,7 +1052,7 @@ static void mrboom_deal_with_skynet_team_mode()
    }
 }
 
-static void mrboom_tick_ai()
+void mrboom_tick_ai()
 {
    for (int i = 0; i < numberOfPlayers(); i++)
    {
